'''
DynamicThresholdIdentification_Validation.py
Created on August 13, 2014
Updated on September 9, 2014
Python Developer: Claudia J. Young (cyoung@usgs.gov)
Python required modules: dbfpy
Description: This python scripts runs and requires the same inputs as the "DynamicThresholdIdentification.py" script, but the output is different. For this script,
              the output is an updated county FIPS raster dataset as an IMG file with the fields "NDVIThrAbv", "PeakNDVIAc", and a field for the area in acres in
              by county for each NDVI value in the associated attributed table. The table can be useful for validation purposes.
'''

#!/usr/bin/env python

# Import system modules
import sys, os, datetime, time, traceback, shutil
from dbfpy import dbf

# arcpy environments and licenses
import arcpy
from arcpy import env
from arcpy.sa import *

arcpy.CheckOutExtension("Spatial")
arcpy.CheckOutExtension("Management")

env.overwriteOutput = True
env.pyramid = None
env.rasterStatistics = None

## Global variables
python_script_dir = os.path.dirname(sys.argv[0])
print python_script_dir

#**************************** MAIN ******************************************
if __name__ == '__main__':

    try:
        if len(sys.argv) != 6:
            print "USAGE: <python> DynamicThresholdIdentification.py [Peak_NDVI] [CountyFIPS] [Output_Dir] [CountyFIPS_fieldname] [USDA_Area_fieldname]"

        else:        
            t1 = datetime.datetime.now()
            print t1.strftime("%Y-%m-%d %H:%M:%S")

            peak_ndvi = sys.argv[1]                 # NDVI data --> G:\Irrigation\PeakNDVI\maxn12_m (2012 data) or G:\Irrigation\Testing\TestPeakNDVI\n07lc06m8182 (2007 data)
            county_fips = sys.argv[2]               # County FIPS data --> G:\Irrigation\County\cntyfips (2012 data) or G:\Irrigation\Testing\TestCounty\cntyfips (2007 data)
            output_dir = sys.argv[3]                # Output directory --> G:\Irrigation\ScratchValidation2012 or G:\Irrigation\ScratchValidation2007
            fips_fieldname = sys.argv[4]            # FIP (2012 data) or FIPS (2007 data)
            usda_irrigarea_fieldname = sys.argv[5]  # IRR_ACR (2012 data) or IRRI_ACRE (2007 data)

            completed_process = 0

            if os.path.exists(peak_ndvi) and os.path.exists(county_fips):

                # Check if output directory exists, if not create it and define as environment workspace
                if not os.path.exists(output_dir):
                    os.makedirs(output_dir)
                env.workspace = output_dir

                # Make copy of input data in the workspace as IMG file to use associated DBF table for the analysis
                peak_ndvi_img = os.path.basename(peak_ndvi) + ".img"
                arcpy.CopyRaster_management(peak_ndvi, peak_ndvi_img)
                print "INFO: %s/%s %s" %(output_dir, peak_ndvi_img, str(arcpy.Exists(peak_ndvi_img)))                
                county_fips_img = os.path.basename(county_fips) + ".img"
                arcpy.CopyRaster_management(county_fips, county_fips_img)
                print "INFO: %s/%s %s" %(output_dir, county_fips_img, str(arcpy.Exists(county_fips_img)))

                # Check if DBF Attribute table from IMG file, of not create it
                peak_ndvi_dbf = peak_ndvi_img + ".vat.dbf"
                county_fips_dbf = county_fips_img + ".vat.dbf"

                # Check if DBF Attribute table from NDVI IMG file, of not create it
                peak_ndvi_dbf = peak_ndvi_img + ".vat.dbf"
                county_fips_dbf = county_fips_img + ".vat.dbf"
                if not os.path.exists(output_dir + os.sep + peak_ndvi_dbf):
                    arcpy.BuildRasterAttributeTable_management(peak_ndvi_img, "Overwrite")
                print "INFO: %s/%s %s" %(output_dir, peak_ndvi_dbf, str(os.path.exists(output_dir + os.sep + peak_ndvi_dbf)))
                print "-" * 20

                # Get fieldnames from the COUNTY FIPS attribute table
                desc_ctyfips_img = arcpy.Describe(county_fips_img)
                desc_ctyfips_dbf_fieldnames = []
                for field in desc_ctyfips_img.fields:
                    desc_ctyfips_dbf_fieldnames.append(field.name)
                print "INFO: County FIPS fieldnames %s" %(str(desc_ctyfips_dbf_fieldnames))
                print "-" * 20

                # Check if user input fieldnames exists in the COUNTY FIPS attribute table
                if fips_fieldname in desc_ctyfips_dbf_fieldnames and usda_irrigarea_fieldname in desc_ctyfips_dbf_fieldnames:

                    # Add new fields which will containt the threshold identification and cumulative peak NDVI area values
                    arcpy.AddField_management(output_dir + os.sep + county_fips_img, "NDVIThrAbv", "LONG")
                    arcpy.CalculateField_management(output_dir + os.sep + county_fips_img, "NDVIThrAbv", "int(999)", "PYTHON_9.3")
                    arcpy.AddField_management(output_dir + os.sep + county_fips_img, "PeakNDVIAc", "FLOAT", 10, 2)
                    arcpy.CalculateField_management(output_dir + os.sep + county_fips_img, "PeakNDVIAc", "float(0.0)", "PYTHON_9.3")

                    # Get cellsize of the peak ndvi data to calculate area in acres
                    x_res = arcpy.GetRasterProperties_management(output_dir + os.sep + peak_ndvi_img, "CELLSIZEX").getOutput(0)
                    y_res = arcpy.GetRasterProperties_management(output_dir + os.sep + peak_ndvi_img, "CELLSIZEY").getOutput(0)
                    print "INFO: peak ndvi resolution (x,y): %s, %s" %(str(x_res), str(y_res))
                    print "-" * 20

                    desc_ndvi = arcpy.Describe(peak_ndvi_img)
                    env.extent = desc_ndvi.extent
                    env.cellSize = peak_ndvi_img
                    env.outputCoordinateSystem = output_dir + os.sep + peak_ndvi_img
                    print "INFO: Environment Settings based on NDVI raster dataset"
                    print "INFO: Extent = " + str(env.extent)
                    print "INFO: Cell Size = " + str(env.cellSize)
                    print "INFO: Output Coordinate System = " + str(env.outputCoordinateSystem.name)
                    print "-" * 20               

                    # Extract from peak NDVI and county FIPS values
                    peak_ndvi_db = dbf.Dbf(output_dir + os.sep + peak_ndvi_dbf)
                    print "INFO: Number of NDVI values found = " + str(len(peak_ndvi_db))
                    ras = arcpy.Raster(output_dir + os.sep + peak_ndvi_img)
                    maxNDVI = int(ras.maximum)
                    minNDVI = int(ras.minimum)
                    print "INFO: Maximum = " + str(maxNDVI)
                    print "INFO: Minimum = " + str(minNDVI)
                    del ras
                    print "-" * 20

                    # look through peak ndvi DBF records to run analysis from the max value to min value
                    i = len(peak_ndvi_db) - 1
                    while i >= 0:
                        rec = peak_ndvi_db[i]
                        peak_ndvi_value = rec["VALUE"]
                        print " " + str(peak_ndvi_value),
                        outExtract = peak_ndvi_img[:-4] + "_" + str(peak_ndvi_value) + ".img"
                        attExtract = ExtractByAttributes(peak_ndvi_img, "VALUE = %s" %(str(peak_ndvi_value)))
                        attExtract.save(outExtract)
                        # Zonal statistics as table to get the count for every county of the value being evaluated
                        outTable = "peakndvi" + str(peak_ndvi_value) + ".dbf"
                        outZSaT = ZonalStatisticsAsTable(county_fips_img, fips_fieldname, outExtract, output_dir + os.sep + outTable, "DATA", "MEAN")

                        # Add fields to calculate area in acres, AREA_ACRES will be use in the peak NDVI analysis, while ACRES_X will contain the area
                        # for each NDVI value for validation/analysis purposes
                        arcpy.AddField_management(outTable, "AREA_ACRES", "FLOAT", 10, 2)
                        arcpy.AddField_management(outTable, "ACRES_" + str(peak_ndvi_value), "FLOAT", 10, 2)

                        # Calculate Area in Acres based on COUNT, pixel resolution, and conversion of m2 to acres (1 m2 = 0.000247105)
                        expression = "getArea(int(!COUNT!)," + x_res + "," + y_res + ")"
                        codeblock = """def getArea(count,x_res,y_res):
                            if count == 0:
                                return float(0.0)
                            else:
                                return float((count * x_res * y_res) * 0.000247105)"""
                        arcpy.CalculateField_management(outTable, "AREA_ACRES", expression, "PYTHON_9.3", codeblock)
                        arcpy.CalculateField_management(outTable, "ACRES_" + str(peak_ndvi_value), expression, "PYTHON_9.3", codeblock)

                        # Join couty fips IMG table with outTable
                        arcpy.JoinField_management(county_fips_img, fips_fieldname, outTable, fips_fieldname, ["AREA_ACRES", "ACRES_" + str(peak_ndvi_value)]) 
                        cursor = arcpy.UpdateCursor(county_fips_img)
                        tmp_peak_ndvi = 0                    
                        for row in cursor:
                            IRRI_ACRES = row.getValue(usda_irrigarea_fieldname)
                            AREA_ACRES = row.AREA_ACRES
                            PeakNDVIAc = row.PeakNDVIAc
                            NDVIThrAbv = row.NDVIThrAbv
                            if IRRI_ACRES != 0: 
                                if PeakNDVIAc + AREA_ACRES > IRRI_ACRES: 
                                    if NDVIThrAbv == 999:
                                        row.setValue("NDVIThrAbv",int(peak_ndvi_value))
                                        row.setValue("PeakNDVIAc",float(PeakNDVIAc + AREA_ACRES))
                                else:
                                    if NDVIThrAbv == 999:
                                        if int(peak_ndvi_value) == int(minNDVI):
                                            if (PeakNDVIAc + AREA_ACRES) > 0:
                                                row.setValue("NDVIThrAbv", int(peak_ndvi_value))
                                                row.setValue("PeakNDVIAc", float(PeakNDVIAc + AREA_ACRES))
                                            else:
                                                row.setValue("PeakNDVIAc",0)
                                        else:
                                            row.setValue("PeakNDVIAc", float(PeakNDVIAc + AREA_ACRES))
                                cursor.updateRow(row)
                        arcpy.DeleteField_management(county_fips_img, ["AREA_ACRES"])
                        arcpy.Delete_management(outExtract)
                        arcpy.Delete_management(outTable)
                        i = i - 1
                    print "\n-----"
                    peak_ndvi_db.close()

                    # Delete IMG NDVI file used as intermediate data
                    arcpy.Delete_management(output_dir + os.sep + peak_ndvi_img)
                    print output_dir + os.sep + peak_ndvi_dbf, os.path.exists(output_dir + os.sep + peak_ndvi_dbf)
                    if os.path.exists(output_dir + os.sep + peak_ndvi_dbf):
                        os.remove(output_dir + os.sep + peak_ndvi_dbf)
                        
                    completed_process = 1
                    
                else:
                    print "ERROR: One or both input fieldnames (%s,%s) does/do not exist in County FIPS attibute table (%s)..." %(fips_fieldname, usda_irrigarea_fieldname, desc_ctyfips_dbf_fieldnames)
                                         
            else:
                print "ERROR: One or both required inputs does not exist..."
                print peak_ndvi, os.path.exists(peak_ndvi)
                print county_fips, os.path.exists(county_fips)                    
            
            t2 = datetime.datetime.now()
            tt = t2 - t1

            if completed_process == 1:
                print "\nINFO: Process for dynamic threshold identification and cell extraction ... COMPLETE"
                print "\n" + t2.strftime("%Y-%m-%d %H:%M:%S")
                print "Processing time (HH:MM:SS): " + str(tt)
                print "*" * 20
            else:
                print "\nINFO: Process for dynamic threshold identification and cell extraction ... INOMPLETE"
                print "\n" + t2.strftime("%Y-%m-%d %H:%M:%S")
                print "*" * 20
            
    except:
        tb = sys.exc_info()[2] 
        exctype, excvalue = sys.exc_info()[:2]
        tbinfo = traceback.format_tb(tb)[0]
        pymsg = "PYTHON ERRORS:\nTraceback Info:\n" + tbinfo + "\nError Info:\n    " + \
                        str(exctype)+ ": " + str(excvalue) + "\n"
        print pymsg
        print "\nINFO: Process for dynamic threshold identification and cell extraction ... INOMPLETE"
        print "-" * 20
        

